#include <kernel.h>
#include <t_syslog.h>
#include <t_stdlib.h>
#include "syssvc/serial.h"
#include "syssvc/syslog.h"
#include "kernel_cfg.h"
#include "sample1.h"
#include "sil.h"

#include "mdcom.h"
#include "mdcom_config.h"

/* mdcom_kernel.c で定義してある memcpy を使う */
extern void *memcpy(void *dest, const void *src, size_t len);

int
filter_even(void *buffer, uint32_t size)
{
	uint32_t value;

	value = *((uint32_t *)buffer);

	if ((value % 2) == 0)  return true;
	return false;
}

#define SHMEM_SIZE 64

void
share_mem_test() {
	int ret;
	void *buffer;
	uint32_t size;

	size = 0;
	
	syslog(LOG_NOTICE, "Share mem test start!.");

	ret = mdcom_smemch_get(MDCOM_SHMEM_1, &buffer);
	if(ret != 0) {
		syslog(LOG_NOTICE, "Get Buffer Error!! %d",ret);
	}

	syslog(LOG_NOTICE, "buffer 0x%x, Size %d.", (unsigned int)buffer, size);

	while(1) {
		syslog(LOG_NOTICE, "0:%d, 1:%d, 2:%d, 3:%d, 4:%d",
			   *((uint32_t *)buffer + 0), *((uint32_t *)buffer + 1),
			   *((uint32_t *)buffer + 2), *((uint32_t *)buffer + 3),
			   *((uint32_t *)buffer + 4));
		dly_tsk(2000);
	}
}

void main_task(intptr_t exinf)
{
	int ret;
	void *buffer;
	mdcom_id_t block_id;
	char msg[] = "hey!";

	syslog(LOG_NOTICE, "MDCOM basic test program starts.");

	syslog(LOG_NOTICE, "Remote mdcom_init");
	dly_tsk(2000);

	do {
		ret = mdcom_init(0x3, -1);
		syslog(LOG_NOTICE, "[Remote] end   mdcom_init. ret = %d", ret);
		dly_tsk(1000);
	} while (ret != MDCOM_SUCCESS);
	assert(ret == 0);

	syslog(LOG_NOTICE, "MDCOMS communications initialized");

	/* phase 1 */
	syslog(LOG_NOTICE, "Remote wait for message");
	/* アロケートされるまで待つ */
	do {
		ret = mdcom_fifoch_wait(MDCOM_CHANNEL_0, -1);
		syslog(LOG_NOTICE, "[Remote] end   mdcom_fifoch_wait. ret = %d", ret);
	} while (ret != MDCOM_SUCCESS);
	assert(ret == 0);

	syslog(LOG_NOTICE, "Event received");

	ret = mdcom_fifoch_dequeue(MDCOM_CHANNEL_0, &block_id);
	syslog(LOG_NOTICE, "mdcom_block_dequeue() %d", ret);
	assert(ret == 0);

	ret = mdcom_fifoch_get(MDCOM_CHANNEL_0, block_id, &buffer);
	assert(ret == 0);

	syslog(LOG_NOTICE, "MSG: %s", (char *)buffer);

	syslog(LOG_NOTICE, "Remote notify ack");
	ret = mdcom_fifoch_notify(MDCOM_CHANNEL_0);
	assert(ret == 0);

	/* phase 2 */
	syslog(LOG_NOTICE, "Remote wait for rpc");
	ret = mdcom_fifoch_wait(MDCOM_CHANNEL_0, -1);
	assert(ret == 0);
	syslog(LOG_NOTICE, "ret = %d",ret);

	syslog(LOG_NOTICE, "received params: %u and %u",
		   *((uint32_t *)buffer), *((uint32_t *)(buffer + 4)));

	*((uint32_t *)buffer) = *((uint32_t *)buffer) * *((uint32_t *)(buffer + 4));
	syslog(LOG_NOTICE, "Remote notify rpc result (%u)", *((uint32_t *)buffer));

	ret = mdcom_fifoch_notify(MDCOM_CHANNEL_0);
	assert(ret == 0);

	/* phase 3 */
	syslog(LOG_NOTICE, "Remote wait for rpc");
	ret = mdcom_fifoch_wait(MDCOM_CHANNEL_0, -1);
	assert(ret == 0);

	syslog(LOG_NOTICE, "received params: %u and %u",
		   *((uint32_t *)buffer), *((uint32_t *)(buffer + 4)));

	*((uint32_t *)buffer) = *((uint32_t *)buffer) * *((uint32_t *)(buffer + 4));
	syslog(LOG_NOTICE, "Remote notify rpc result (%u)", *((uint32_t *)buffer));
//OK
	ret = mdcom_fifoch_notify(MDCOM_CHANNEL_0);
	assert(ret == 0);

	ret = mdcom_fifoch_wait(MDCOM_CHANNEL_0, -1);
	assert(ret == 0);

	syslog(LOG_NOTICE, "Remote mdcom_fifoch_free");
	ret = mdcom_fifoch_free(MDCOM_CHANNEL_0, block_id);
	assert(ret == 0);
//OK
	/* phase 4 */
	syslog(LOG_NOTICE, "Notify a message to the Master by channel 1");
	ret = mdcom_fifoch_alloc(MDCOM_CHANNEL_1, &block_id);
	assert(ret == 0);

	ret = mdcom_fifoch_get(MDCOM_CHANNEL_1, block_id, &buffer);
	assert(ret == 0);

	memcpy(buffer, (void *)msg, sizeof(msg));

	ret = mdcom_fifoch_enqueue(MDCOM_CHANNEL_1, block_id);
	assert(ret == 0);

	ret = mdcom_fifoch_notify(MDCOM_CHANNEL_1);
	assert(ret == 0);
//OK

	/* phase 5 */
	syslog(LOG_NOTICE, "Notify several messages with a filter on");

	ret = mdcom_fifoch_wait(MDCOM_CHANNEL_0, -1);
	assert(ret == 0);

	ret = mdcom_fifoch_alloc(MDCOM_CHANNEL_0, &block_id);
	assert(ret == 0);

	ret = mdcom_fifoch_get(MDCOM_CHANNEL_0, block_id, &buffer);
	assert(ret == 0);

	*((uint32_t *)buffer) = 1;

	ret = mdcom_fifoch_enqueue(MDCOM_CHANNEL_0, block_id);

	if(ret != 0) syslog(LOG_NOTICE, "data %u filtered",*((uint32_t *)buffer));
	ret = mdcom_fifoch_alloc(MDCOM_CHANNEL_0, &block_id);
	assert(ret == 0);

	ret = mdcom_fifoch_get(MDCOM_CHANNEL_0, block_id, &buffer);
	assert(ret == 0);

	*((uint32_t *)buffer) = 2;

	ret = mdcom_fifoch_enqueue(MDCOM_CHANNEL_0, block_id);
	if(ret != 0) syslog(LOG_NOTICE, "data %u filtered",*((uint32_t *)buffer));

	ret = mdcom_fifoch_alloc(MDCOM_CHANNEL_0, &block_id);
	assert(ret == 0);

	ret = mdcom_fifoch_get(MDCOM_CHANNEL_0, block_id, &buffer);
	assert(ret == 0);

	*((uint32_t *)buffer) = 3;

	ret = mdcom_fifoch_enqueue(MDCOM_CHANNEL_0, block_id);
	if(ret != 0) syslog(LOG_NOTICE, "data %u filtered",*((uint32_t *)buffer));

	ret = mdcom_fifoch_notify(MDCOM_CHANNEL_0);
	assert(ret == 0);

	share_mem_test();
}
