#include <stdio.h>
#include <string.h>
#include <assert.h>
#include <unistd.h>
#include "mdcom.h"
#include "mdcom_config.h"

bool_t
filter_even(void *buffer, uint32_t size)
{
	uint32_t value;

	value = *((uint32_t *)buffer);

	if ((value % 2) == 0)  return true;
	return false;
}

#define SHMEM_SIZE 64

void
share_mem_test() {
	int ret;
	void *buffer;
	uint32_t size;
	int i;

	size = 0;
	
	printf("Share mem test start!.\n");
	fflush(stdout);

	ret = mdcom_smemch_get(MDCOM_SHMEM_1, &buffer);
	if(ret != 0) {
		printf("Get Buffer Error!! %d\n",ret);
		fflush(stdout);
	}

	printf("buffer 0x%p, Size %d.\n", buffer, size);
	fflush(stdout);

	while(1) {
		for(i = 0; i < SHMEM_SIZE/8; i++){
			printf("%d : %d, ", i, *((uint32_t *)buffer + i));
		}
		printf("\n");
		sleep(2);
	}
}

int main()
{
	int ret;
	void *buffer;
	mdcom_id_t block_id;
	char msg[] = "hey!";

	printf("MDCOM basic test program starts.\n");
	printf("REMOTE mdcom_init()\n");
	do {
		ret = mdcom_init(0x3, -1);
		printf("["MDCOM_DOMAIN_NAME"] end   mdcom_init. ret = %d\n", ret);
		sleep(2);
	} while (ret != MDCOM_SUCCESS);
	assert(ret == 0);

	printf("Dual-OS communications initialized\n");

	/* phase 1 */
	printf("REMOTE wait for message\n");
	/* アロケートされるまで待つ */
	do {
		ret = mdcom_fifoch_wait(MDCOM_CHANNEL_0, 0);
		printf("["MDCOM_DOMAIN_NAME"] end   mdcom_fifoch_wait phase 1. ret = %d\n", ret);
		fflush(stdout);
	} while (ret != MDCOM_SUCCESS);
	assert(ret == 0);

	printf("Event received\n");

	ret = mdcom_fifoch_dequeue(MDCOM_CHANNEL_0, &block_id);
	printf("dualoscom_block_dequeue() %d\n", ret);
	assert(ret == 0);

	ret = mdcom_fifoch_get(MDCOM_CHANNEL_0, block_id, &buffer);
	assert(ret == 0);

	printf("MSG: %s\n", (char *)buffer);

	printf("REMOTE notify ack\n");
	ret = mdcom_fifoch_notify(MDCOM_CHANNEL_0);
	assert(ret == 0);

	/* phase 2 */
	printf("REMOTE wait for rpc\n");
	ret = mdcom_fifoch_wait(MDCOM_CHANNEL_0, 0);
	assert(ret == 0);

	printf("received params: %u and %u\n",
		   *((uint32_t *)buffer), *((uint32_t *)(buffer + 4)));

	*((uint32_t *)buffer) = *((uint32_t *)buffer) * *((uint32_t *)(buffer + 4));
	printf("REMOTE notify rpc result (%u)\n", *((uint32_t *)buffer));

	ret = mdcom_fifoch_notify(MDCOM_CHANNEL_0);
	assert(ret == 0);


	/* phase 3 */
	printf("REMOTE wait for rpc\n");
	ret = mdcom_fifoch_wait(MDCOM_CHANNEL_0, 0);
	assert(ret == 0);

	printf("received params: %u and %u\n",
		   *((uint32_t *)buffer), *((uint32_t *)(buffer + 4)));

	*((uint32_t *)buffer) = *((uint32_t *)buffer) * *((uint32_t *)(buffer + 4));
	printf("REMOTE notify rpc result (%u)\n", *((uint32_t *)buffer));

	ret = mdcom_fifoch_notify(MDCOM_CHANNEL_0);
	assert(ret == 0);

	ret = mdcom_fifoch_wait(MDCOM_CHANNEL_0, 0);
	assert(ret == 0);

	printf("REMOTE mdcom_fifoch_free\n");
	ret = mdcom_fifoch_free(MDCOM_CHANNEL_0, block_id);
	assert(ret == 0);


	/* phase 4 */
	printf("Notify a message to the RTOS by channel 1\n");
	ret = mdcom_fifoch_alloc(MDCOM_CHANNEL_1, &block_id);
	assert(ret == 0);

	ret = mdcom_fifoch_get(MDCOM_CHANNEL_1, block_id, &buffer);
	assert(ret == 0);

	memcpy(buffer, (void *)msg, sizeof(msg));

	ret = mdcom_fifoch_enqueue(MDCOM_CHANNEL_1, block_id);
	assert(ret == 0);

	ret = mdcom_fifoch_notify(MDCOM_CHANNEL_1);
	assert(ret == 0);


	/* phase 5 */
	printf("Notify several messages with a filter on\n");

	ret = mdcom_fifoch_wait(MDCOM_CHANNEL_0, 0);
	assert(ret == 0);

	ret = mdcom_fifoch_alloc(MDCOM_CHANNEL_0, &block_id);
	assert(ret == 0);

	ret = mdcom_fifoch_get(MDCOM_CHANNEL_0, block_id, &buffer);
	assert(ret == 0);

	*((uint32_t *)buffer) = 1;

	ret = mdcom_fifoch_enqueue(MDCOM_CHANNEL_0, block_id);
	if(ret != 0) printf("data %u filtered\n",*((uint32_t *)buffer));

	ret = mdcom_fifoch_alloc(MDCOM_CHANNEL_0, &block_id);
	assert(ret == 0);

	ret = mdcom_fifoch_get(MDCOM_CHANNEL_0, block_id, &buffer);
	assert(ret == 0);

	*((uint32_t *)buffer) = 2;

	ret = mdcom_fifoch_enqueue(MDCOM_CHANNEL_0, block_id);
	if(ret != 0) printf("data %u filtered\n",*((uint32_t *)buffer));

	ret = mdcom_fifoch_alloc(MDCOM_CHANNEL_0, &block_id);
	assert(ret == 0);

	ret = mdcom_fifoch_get(MDCOM_CHANNEL_0, block_id, &buffer);
	assert(ret == 0);

	*((uint32_t *)buffer) = 3;

	ret = mdcom_fifoch_enqueue(MDCOM_CHANNEL_0, block_id);
	if(ret != 0) printf("data %u filtered\n",*((uint32_t *)buffer));

	ret = mdcom_fifoch_notify(MDCOM_CHANNEL_0);
	assert(ret == 0);

	share_mem_test();

	return 0;
}
