/*
 *  @(#) $Id: mdcom_target_driver.h 897 2017-05-24 07:30:19Z ertl-honda $
 */
#include <linux/imx_sema4.h>

/*
 * ARCHITECTURE
 */
#define __TARGET_ARCH_ARM 8

/*
 * SHARED MEMORY (allocated in kernel parameters)
 */
#define SREGION_PHYS_ADDRESS	0xBCC00000UL
#define SREGION_SIZE		    0x00200000UL /* 2MB */
#define SREGION_WRITECOMBINE

/*
 *  Use MU as Giant Lock
 */
#define MDCOM_MUTEX_SEMA4_ID  15

/*
 * Errors
 */
#define MDCOM_SUCCESS         (0) /* function returned successfully */
#define MDCOM_ERR_SEMAPHORE   (1) /* semaphore handling error */

/*
 * MAX VALUES (can be adjusted)
 */
#define MAX_NUM_CHANNELS   1024
#define MAX_NUM_GROUPS     64
#define MAX_PENDING_OFFSET (SHREGION_SIZE - MAX_NUM_CHANNELS*8)
#define MDCOM_MAX_NUM_SEMA4	SEMA4_NUM_GATES		/* 16 */

/*
 *  SEMA4 Driver
 */
static struct imx_sema4_mutex *p_imx_sema4_mutex;
static bool imx_sema4_lockd;

static int
_target_lock_init(void)
{
	/* Create a core mutex */
	p_imx_sema4_mutex = imx_sema4_mutex_create(0, MDCOM_MUTEX_SEMA4_ID);

	imx_sema4_lockd = false;

	if (NULL == p_imx_sema4_mutex) {
		return MDCOM_ERR_SEMAPHORE;
	}
	else {
		return MDCOM_SUCCESS;
	}
}

static int
_target_lock_deinit(void)
{
	imx_sema4_lockd = false;

	/* Destroy the core mutex */
	if (0 == imx_sema4_mutex_destroy(p_imx_sema4_mutex)) {
		return MDCOM_SUCCESS;
	}
	else {
		return MDCOM_ERR_SEMAPHORE;
	}
}

static int
_target_lock_trylock(void)
{
	if(imx_sema4_lockd) {
		return MDCOM_ERR_SEMAPHORE;
	}
	  
	if (SEMA4_A9_LOCK == imx_sema4_mutex_trylock(p_imx_sema4_mutex)) {
		imx_sema4_lockd = true;
		return MDCOM_SUCCESS;
	}

	return MDCOM_ERR_SEMAPHORE;
}

static int
_target_lock_unlock(void)
{
	if(!imx_sema4_lockd) {
		return MDCOM_ERR_SEMAPHORE;
	}

	if (0 == imx_sema4_mutex_unlock(p_imx_sema4_mutex)) {
		imx_sema4_lockd = false;
		return MDCOM_SUCCESS;
	}

	return MDCOM_ERR_SEMAPHORE;
}

/*
 *  MU Driver
 */
static void
_target_int_send(void)
{
	mdcom_triger_cpu_to_cpu_interrupt();
}

/* Forward Reference */
irqreturn_t mdcom_irq_handler(int irq, void *dev_id);

void
mdcom_mu_callback(int irq, void *param)
{
    (void)mdcom_irq_handler(irq, param);
}

static void
_target_int_init(void)
{
	mdcom_register_callback(mdcom_mu_callback);
}

static void
_target_int_exit(void)
{
	mdcom_unregister_callback();
}

static void
_target_int_clear(void)
{
	/* do nothing */
}
