/*
 *  TOPPERS/MDCOM
 *      Toyohashi Open Platform for Embedded Real-Time Systems/
 *      Multi Domain Communication Module
 *
 *  Copyright (C) 2016-2018 by Embedded and Real-Time Systems Laboratory
 *     Graduate School of Information Science, Nagoya University, JAPAN
 *
 *  上記著作権者は，以下の(1)〜(4)の条件を満たす場合に限り，本ソフトウェ
 *  ア（本ソフトウェアを改変したものを含む．以下同じ）を使用・複製・改
 *  変・再配布（以下，利用と呼ぶ）することを無償で許諾する．
 *  (1) 本ソフトウェアをソースコードの形で利用する場合には，上記の著作
 *      権表示，この利用条件および下記の無保証規定が，そのままの形でソー
 *      スコード中に含まれていること．
 *  (2) 本ソフトウェアを，ライブラリ形式など，他のソフトウェア開発に使
 *      用できる形で再配布する場合には，再配布に伴うドキュメント（利用
 *      者マニュアルなど）に，上記の著作権表示，この利用条件および下記
 *      の無保証規定を掲載すること．
 *  (3) 本ソフトウェアを，機器に組み込むなど，他のソフトウェア開発に使
 *      用できない形で再配布する場合には，次のいずれかの条件を満たすこ
 *      と．
 *    (a) 再配布に伴うドキュメント（利用者マニュアルなど）に，上記の著
 *        作権表示，この利用条件および下記の無保証規定を掲載すること．
 *    (b) 再配布の形態を，別に定める方法によって，TOPPERSプロジェクトに
 *        報告すること．
 *  (4) 本ソフトウェアの利用により直接的または間接的に生じるいかなる損
 *      害からも，上記著作権者およびTOPPERSプロジェクトを免責すること．
 *      また，本ソフトウェアのユーザまたはエンドユーザからのいかなる理
 *      由に基づく請求からも，上記著作権者およびTOPPERSプロジェクトを
 *      免責すること．
 *
 *  本ソフトウェアは，無保証で提供されているものである．上記著作権者お
 *  よびTOPPERSプロジェクトは，本ソフトウェアに関して，特定の使用目的
 *  に対する適合性も含めて，いかなる保証も行わない．また，本ソフトウェ
 *  アの利用により直接的または間接的に生じたいかなる損害に関しても，そ
 *  の責任を負わない．
 *
 * [target_common.h]
 *
 * This file contains imx8_amp target-dependent constant definitions
 * used both monitor and user program.
 *
 * @(#) $Id: mdcom_target_common.h 902 2017-06-02 11:05:58Z ertl-honda $  
 */
#ifndef _MDCOM_TARGET_COMMON_H_
#define _MDCOM_TARGET_COMMON_H_

/*
 *  Use SEMA4 as Giant Lock
 */
#define MDCOM_USE_DRIVER_LOCK
#define MDCOM_MUTEX_SEMA4_ID  15

/*
 * SHMEM address
 *   Core common memory is 0x40000000-0xFFFFFFFF.
 *   u-boot uses 0xFD00000-0xFFFFFFFF
 *   This implemet assgined SHMEM at bottom of Core common memory.
 */
#define MDCOM_SHMEM_ADDRESS  0xBCC00000
#define MDCOM_SHMEM_SIZE     0x00200000

/*
 *  Register Defintion
 */
#define SET_MU_GIR0     1u << 16
#define SET_MU_GIR1     1u << 17
#define SET_MU_GIR2     1u << 18
#define SET_MU_GIR3     1u << 19
#define SET_MU_IRQ_REQ  SET_MU_GIR0
#define READ_MU_IRQ_REQ(val)    val & SET_MU_IRQ_REQ

#define ENA_MU_GIE0     1u << 28
#define ENA_MU_GIE1     1u << 29
#define ENA_MU_GIE2     1u << 30
#define ENA_MU_GIE3     1u << 31
#define ENA_MU_IRQ      ENA_MU_GIE0

#define CLR_MU_GIP0     1u << 28
#define CLR_MU_GIP1     1u << 29
#define CLR_MU_GIP2     1u << 30
#define CLR_MU_GIP3     1u << 21
#define CLR_MU_IRQ      CLR_MU_GIP0

#define SEMA4_UNLOCK    0x00
#define SEMA4_CA53_LOCK 0x01
#define SEMA4_CM4_LOCK  0x02
#define SEMA4_GATE_MASK 0x03

#ifdef MDCOM_TARGET_CA53

#define MU_A_BASE_ADDR  0x30AA0000
#define MU_ASR          (MU_A_BASE_ADDR + 0x0020)
#define MU_ACR          (MU_A_BASE_ADDR + 0x0024)

#define MU_B_BASE_ADDR  0x30AB0000
#define MU_BSR          (MU_B_BASE_ADDR + 0x0020)
#define MU_BCR          (MU_B_BASE_ADDR + 0x0024)

#define MU_SR           MU_ASR
#define MU_CR           MU_ACR

#define TARGET_MU_INT_ID    (88 + 32)

#define SEMA4_BASE_ADDR 0x30AC0000
#define SEMA4_CPU_LOCK  SEMA4_CA53_LOCK

#define CCM_BASE_ADDR   0x30380000
#define CCM_CCGR33      (CCM_BASE_ADDR + 0x4210)
#define CCM_CCGR61      (CCM_BASE_ADDR + 0x43D0)
#define CCM_CCGR62      (CCM_BASE_ADDR + 0x43E0)

#elif defined(MDCOM_TARGET_CM4)

#define MU_A_BASE_ADDR  0x30AA0000
#define MU_ASR          (MU_A_BASE_ADDR + 0x0020)
#define MU_ACR          (MU_A_BASE_ADDR + 0x0024)

#define MU_B_BASE_ADDR  0x30AB0000
#define MU_BSR          (MU_B_BASE_ADDR + 0x0020)
#define MU_BCR          (MU_B_BASE_ADDR + 0x0024)

#define MU_SR           MU_BSR
#define MU_CR           MU_BCR

#define TARGET_MU_INT_ID    (97 + 16)

#define SEMA4_BASE_ADDR 0x30AC0000
#define SEMA4_CPU_LOCK  SEMA4_CM4_LOCK

#define CCM_BASE_ADDR   0x30380000
#define CCM_CCGR33      (CCM_BASE_ADDR + 0x4210)
#define CCM_CCGR61      (CCM_BASE_ADDR + 0x43D0)
#define CCM_CCGR62      (CCM_BASE_ADDR + 0x43E0)

#else  /* MDCOM_TARGET_CM4 */
#error "CA53 or CM4"
#endif /* MDCOM_TARGET_CA53 */

#define SEMA4_CP0INE    SEMA4_BASE_ADDR + 0x40
#define SEMA4_CP1INE    SEMA4_BASE_ADDR + 0x48
#define SEMA4_CP0NTF    SEMA4_BASE_ADDR + 0x80
#define SEMA4_CP1NTF    SEMA4_BASE_ADDR + 0x88

/*
 * MU/SEMA4 Driver for TOPPERS OS
 */
#if MDCOM_TYPE_API == MDCOM_TYPE_API_TOPPERS

#define SEND_IRQ_OK 1
#define SEND_IRQ_NG 0

#define ELOCKED     -6
#define EUNLOCKED   -7

/*
 *  Wrie or data to device register
 */
static inline void
__mdcom_write_or(uint32_t *ptr, uint32_t data)
{
	uint32_t val;

	val = sil_rew_mem((uint32_t *)ptr);
	val |= data;
    
	sil_wrw_mem((uint32_t *)ptr, val);
}

/*
 *  Check MU interrupt request
 */
static inline uint32_t
__mdcom_check_irq_req(void)
{
	uint32_t val;

	val = READ_MU_IRQ_REQ(sil_rew_mem((uint32_t *)MU_CR));
	return val;
}

/*
 *  Send MU interrupt request
 */
static inline void
__mdcom_send_irq(void)
{
	/* Wait until MU interrupt request becom 0 */
	while (__mdcom_check_irq_req());

	__mdcom_data_sync();

	/* Send interrupt request */
	__mdcom_write_or((uint32_t *)MU_CR, SET_MU_IRQ_REQ);
}

/*
 *  MU initialization
 */
static inline void
__mdcom_kernel_init_init()
{
	/* Enable clock always */
	sil_wrw_mem((uint32_t *)CCM_CCGR33, 0x00000003);
	/* Enable my MU interrupt */
	__mdcom_write_or((uint32_t *)MU_CR, ENA_MU_IRQ);
}

/*
 *  Clear MU interrupt
 */
static inline void
__mdcom_kernel_clear_int(void)
{
	/* Clear my MU interrupt pending bit */
	__mdcom_write_or((uint32_t *)MU_SR, CLR_MU_IRQ);
}

/*
 *  SEMA4 initialization 
 */
static inline void
__mdcom_kernel_mutex_init(void)
{
	uint8_t gate_val;
	uint32_t sema4_id = MDCOM_MUTEX_SEMA4_ID;    

	/* Enable clock always */
	sil_wrw_mem((uint32_t *)CCM_CCGR61, 0x00000003);
	sil_wrw_mem((uint32_t *)CCM_CCGR62, 0x00000003);

	/* Initialize gate value */
	gate_val = sil_reb_mem((uint8_t *)(SEMA4_BASE_ADDR + sema4_id));
	gate_val &= (~SEMA4_GATE_MASK);
	gate_val |= SEMA4_UNLOCK;
	sil_wrb_mem((uint8_t *)(SEMA4_BASE_ADDR + sema4_id), gate_val);    
}

/*
 *  Try lock mutex
 */
static inline bool_t
__mdcom_kernel_mutex_trylock(uint32_t *p_lock)
{
	uint8_t gate_val;
	uint32_t sema4_id = MDCOM_MUTEX_SEMA4_ID;
	bool_t ret = false;
	SIL_PRE_LOC;

#if MDCOM_TYPE_OS == MDCOM_TYPE_OS_ASP
	SIL_LOC_INT();
#else /* MDCOM_TYPE_OS == MDCOM_TYPE_OS_FMP */ 
	SIL_LOC_SPN();
#endif /* MDCOM_TYPE_OS == MDCOM_TYPE_OS_ASP */
    
	/* Lock SEMA4 */
	do {
		/* Read and Write gate value */
		gate_val = sil_reb_mem((uint8_t *)(SEMA4_BASE_ADDR + sema4_id));
		gate_val &= (~SEMA4_GATE_MASK);
		gate_val |= SEMA4_CPU_LOCK;
		sil_wrb_mem((uint8_t *)(SEMA4_BASE_ADDR + sema4_id), gate_val);
		/* Check lockec */
		gate_val = sil_reb_mem((uint8_t *)(SEMA4_BASE_ADDR + sema4_id));
		gate_val &= SEMA4_GATE_MASK;
	} while(gate_val != SEMA4_CPU_LOCK);

	/* Check free then lock */
	if(*p_lock == 0) {
		*p_lock = 1;
		ret = true;
	}

	/* Unlock SEMA4 */
	gate_val = sil_reb_mem((uint8_t *)(SEMA4_BASE_ADDR + sema4_id));
	gate_val &= (~SEMA4_GATE_MASK);
	gate_val |= SEMA4_UNLOCK;
	sil_wrb_mem((uint8_t *)(SEMA4_BASE_ADDR + sema4_id), gate_val);
    
#if MDCOM_TYPE_OS == MDCOM_TYPE_OS_ASP
	SIL_UNL_INT();
#else /* MDCOM_TYPE_OS == MDCOM_TYPE_OS_FMP */ 
	SIL_UNL_SPN();
#endif /* MDCOM_TYPE_OS == MDCOM_TYPE_OS_ASP */

	return ret;    
}

/*
 *  Unlock mutex 
 */
static inline void
__mdcom_kernel_mutex_unlock(uint32_t *p_lock)
{
	*p_lock = 0;
}
#endif /* MDCOM_TYPE_API == MDCOM_TYPE_API_TOPPERS */

#endif /* _MDCOM_TARGET_DEF_H_ */
